@@ -7,6 +7,7 @@ no if $] >= 5.018, warnings => 'experimental::smartmatch';
 use GooCanvas2;
 use Glib 1.220 qw(TRUE FALSE);    # To get TRUE and FALSE
 use HTML::Entities;
+use Carp;
 use Readonly;
 Readonly my $_100_PERCENT       => 100;
 Readonly my $_360_DEGREES       => 360;
@@ -14,8 +15,9 @@ Readonly my $FULLPAGE_OCR_SCALE => 0.8;
 my $SPACE = q{ };
 my $EMPTY = q{};
 my $device;
+my %old_idles;
 
-our $VERSION = '2.6.2';
+our $VERSION = '2.6.6';
 
 use Glib::Object::Subclass GooCanvas2::Canvas::, signals => {
     'zoom-changed' => {
@@ -110,18 +112,27 @@ sub SET_PROPERTY {
     return;
 }
 
-sub add_text {
-    my ( $self, $page, $edit_callback ) = @_;
-    my $root = $self->get_root_item;
-    if ( not defined $page->{w} ) {
-
-        # quotes required to prevent File::Temp object being clobbered
-        my $pixbuf = Gtk3::Gdk::Pixbuf->new_from_file("$page->{filename}");
-        $page->{w} = $pixbuf->get_width;
-        $page->{h} = $pixbuf->get_height;
+sub set_text {
+    my ( $self, $page, $edit_callback, $idle ) = @_;
+    if ( not defined $idle ) {
+        $idle = TRUE;
     }
-    $self->{pixbuf_size} = { width => $page->{w}, height => $page->{h} };
-    $self->set_bounds( 0, 0, $page->{w}, $page->{h} );
+    my $root;
+    if (%old_idles) {
+        while ( my ( $box, $source ) = each %old_idles ) {
+            Glib::Source->remove($source);
+            delete $old_idles{$box};
+        }
+    }
+    $root = GooCanvas2::CanvasGroup->new;
+    my ( $width, $height ) = $page->get_size;
+    my ( $xres,  $yres )   = $page->get_resolution;
+
+    # Commenting out the scaling factor, as it segfaults in GooCanvas2 v0.06
+    # $root->set_scale($yres / $xres, 1);
+    $self->set_root_item($root);
+    $self->{pixbuf_size} = { width => $width, height => $height };
+    $self->set_bounds( 0, 0, $width, $height );
 
     my $style        = $self->get_style_context;
     my $color_string = $style->get_color('normal')->to_string;
@@ -132,7 +143,26 @@ sub add_text {
 
     # Attach the text to the canvas
     for my $box ( @{ $page->boxes } ) {
-        _boxed_text( $root, $box, [ 0, 0, 0 ], $edit_callback, $color_hex );
+        my %options = (
+            root           => $root,
+            box            => $box,
+            transformation => [ 0, 0, 0 ],
+            edit_callback  => $edit_callback,
+            text_color     => $color_hex,
+            idle           => $idle,
+        );
+        if ($idle) {
+            $old_idles{$box} = Glib::Idle->add(
+                sub {
+                    _boxed_text(%options);
+                    delete $old_idles{$box};
+                    return Glib::SOURCE_REMOVE;
+                }
+            );
+        }
+        else {
+            _boxed_text(%options);
+        }
     }
     return;
 }
@@ -144,10 +174,7 @@ sub get_pixbuf_size {
 
 sub clear_text {
     my ($self) = @_;
-    my $root = $self->get_root_item;
-    if ( $root->get_n_children > 0 ) {
-        $root->remove_child(0);
-    }
+    $self->set_root_item( GooCanvas2::CanvasGroup->new );
     delete $self->{pixbuf_size};
     return;
 }
@@ -193,7 +220,13 @@ sub get_offset {
 # Draw text on the canvas with a box around it
 
 sub _boxed_text {
-    my ( $root, $box, $transformation, $edit_callback, $text_color ) = @_;
+    my (%options)      = @_;
+    my $root           = $options{root};
+    my $box            = $options{box};
+    my $transformation = $options{transformation};
+    my $edit_callback  = $options{edit_callback};
+    my $text_color     = $options{text_color};
+    my $idle           = $options{idle};
     my ( $rotation, $x0, $y0 ) = @{$transformation};
     my ( $x1, $y1, $x2, $y2 ) = @{ $box->{bbox} };
     my $x_size = abs $x2 - $x1;
@@ -259,6 +292,11 @@ sub _boxed_text {
         );
         my $angle  = -( $textangle + $rotation ) % $_360_DEGREES;
         my $bounds = $text->get_bounds;
+        if ( ( $bounds->x2 - $bounds->x1 ) == 0 ) {
+            Glib->warning( __PACKAGE__,
+                "text $box->{text} has no width, skipping" );
+            return;
+        }
         my $scale =
           ( $angle ? $y_size : $x_size ) / ( $bounds->x2 - $bounds->x1 );
 
@@ -284,8 +322,26 @@ sub _boxed_text {
     }
     if ( $box->{contents} ) {
         for my $box ( @{ $box->{contents} } ) {
-            _boxed_text( $g, $box, [ $textangle + $rotation, $x1, $y1 ],
-                $edit_callback, $text_color );
+            my %noptions = (
+                root           => $g,
+                box            => $box,
+                transformation => [ $textangle + $rotation, $x1, $y1 ],
+                edit_callback  => $edit_callback,
+                text_color     => $text_color,
+                idle           => $idle,
+            );
+            if ($idle) {
+                $old_idles{$box} = Glib::Idle->add(
+                    sub {
+                        _boxed_text(%noptions);
+                        delete $old_idles{$box};
+                        return Glib::SOURCE_REMOVE;
+                    }
+                );
+            }
+            else {
+                _boxed_text(%noptions);
+            }
         }
     }
 
