@@ -13,7 +13,7 @@ Readonly my $COL_CHECKBOX => 4;
 
 my %types;
 
-our $VERSION = '2.6.2';
+our $VERSION = '2.6.6';
 my $SPACE    = q{ };
 my $HEXREGEX = qr{^(.*)           # start of message
                   \b0x[[:xdigit:]]+\b # hex (e.g. address)
@@ -32,6 +32,10 @@ sub INIT_INSTANCE {
     );
     $self->set_position('center-on-parent');
     my $vbox = $self->get_content_area;
+
+    # to ensure dialog can't grow too big if we have too many messages
+    my $scwin = Gtk3::ScrolledWindow->new;
+    $vbox->pack_start( $scwin, TRUE, TRUE, 0 );
     $self->{grid}             = Gtk3::Grid->new;
     $self->{grid_rows}        = 0;
     $self->{stored_responses} = [];
@@ -57,14 +61,12 @@ sub INIT_INSTANCE {
     $self->{cb}->set_halign('center');
     $self->{grid}
       ->attach( $self->{cb}, $COL_CHECKBOX, $self->{grid_rows}, 1, 1 );
-    $vbox->pack_start( $self->{grid}, TRUE, TRUE, 0 );
+    $scwin->add( $self->{grid} );
     $self->{cb}->signal_connect(
         toggled => sub {
-
-            if ( $self->{cb}->get_active ) {
-                for my $cb ( $self->_list_checkboxes ) {
-                    $cb->set_active(TRUE);
-                }
+            my $state = $self->{cb}->get_active;
+            for my $cb ( $self->_list_checkboxes ) {
+                $cb->set_active($state);
             }
         }
     );
@@ -96,6 +98,7 @@ sub add_row {
 
     if ( $options{'store-response'} ) {
         my $button = Gtk3::CheckButton->new();
+        $button->signal_connect( toggled => \&_checkbutton_consistency, $self );
         $button->set_halign('center');
         $self->{grid}
           ->attach( $button, $COL_CHECKBOX, $self->{grid_rows} - 1, 1, 1 );
@@ -103,6 +106,7 @@ sub add_row {
             $self->{stored_responses}[ $self->{grid_rows} - 1 ] =
               $options{'stored-responses'};
         }
+        _checkbutton_consistency( $button, $self );
     }
     if ( $self->{grid_rows} > 2 ) {
         $self->{cbl}->set_label( __("Don't show these messages again") );
@@ -110,6 +114,28 @@ sub add_row {
     return;
 }
 
+sub _checkbutton_consistency {
+    my ( $widget, $self ) = @_;
+    my $state;
+    for my $cb ( $self->_list_checkboxes ) {
+        if ( not defined $state ) {
+            $state = $cb->get_active;
+        }
+        elsif ( $state != $cb->get_active ) {
+            undef $state;
+            last;
+        }
+    }
+    if ( defined $state ) {
+        $self->{cb}->set_inconsistent(FALSE);
+        $self->{cb}->set_active($state);
+    }
+    else {
+        $self->{cb}->set_inconsistent(TRUE);
+    }
+    return;
+}
+
 sub add_message {
     my ( $self, %options ) = @_;
 
