@@ -19,6 +19,7 @@ use List::MoreUtils qw(first_index);
 use Readonly;
 Readonly my $PAPER_TOLERANCE  => 1;
 Readonly my $OPTION_TOLERANCE => 0.001;
+Readonly my $INFINITE         => -1;
 
 my (
     $MAX_PAGES,        $MAX_INCREMENT, $DOUBLE_INCREMENT,
@@ -286,9 +287,16 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::, signals => {
         'name of current cursor',                                 # blurb
         [qw/readable writable/]                                   # flags
     ),
+    Glib::ParamSpec->boolean(
+        'ignore-duplex-capabilities',    # name
+        'Ignore duplex capabilities',    # nick
+        'Ignore duplex capabilities',    # blurb
+        FALSE,                           # default_value
+        [qw/readable writable/]          # flags
+    ),
   ];
 
-our $VERSION = '2.6.2';
+our $VERSION = '2.6.6';
 
 my ( $d_sane, $logger );
 my $SANE_NAME_SCAN_TL_X   = SANE_NAME_SCAN_TL_X;
@@ -369,6 +377,10 @@ sub INIT_INSTANCE {
                 $bscanall->set_active(TRUE);
             }
             else {
+                # if spin button is already $value, but pages = all is selected,
+                # then the callback will not fire to activate # pages, so doing
+                # it here
+                $bscannum->set_active(TRUE);
                 $spin_buttonn->set_value($value);
             }
 
@@ -529,6 +541,9 @@ sub INIT_INSTANCE {
             my ( $widget, $value ) = @_;
             $self->set( 'page-number-increment',
                 $value eq 'facing' ? $DOUBLE_INCREMENT : -$DOUBLE_INCREMENT );
+            if ( $value eq 'facing' ) {
+                $self->set( 'num-pages', 0 );
+            }
         }
     );
     $self->{combobs}->set_tooltip_text(
@@ -871,6 +886,28 @@ sub _build_profile_table {
     return;
 }
 
+sub _set_side_to_scan {
+    my ( $self, $name, $newval ) = @_;
+    $self->{$name} = $newval;
+    $self->signal_emit( 'changed-side-to-scan', $newval );
+    $self->{combobs}->set_active( $newval eq 'facing' ? 0 : 1 );
+    my $slist = $self->get('document');
+    if ( defined $slist ) {
+        my $possible = $slist->pages_possible(
+            $self->get('page-number-start'),
+            $self->get('page-number-increment')
+        );
+        my $requested = $self->get('num-pages');
+        if ( $possible != $INFINITE
+            and ( $requested == 0 or $requested > $possible ) )
+        {
+            $self->set( 'num-pages', $possible );
+            $self->set( 'max-pages', $possible );
+        }
+    }
+    return;
+}
+
 sub SET_PROPERTY {
     my ( $self, $pspec, $newval ) = @_;
     my $name   = $pspec->get_name;
@@ -924,6 +961,10 @@ sub SET_PROPERTY {
                         'row-changed' => sub { $self->update_start_page } );
                 }
             }
+            when ('ignore_duplex_capabilities') {
+                $self->{$name} = $newval;
+                $self->_flatbed_or_duplex_callback;
+            }
             when ('num_pages') {
                 $self->_set_num_pages( $name, $newval );
             }
@@ -936,9 +977,7 @@ sub SET_PROPERTY {
                 $self->signal_emit( 'changed-page-number-increment', $newval )
             }
             when ('side_to_scan') {
-                $self->{$name} = $newval;
-                $self->signal_emit( 'changed-side-to-scan', $newval );
-                $self->{combobs}->set_active( $newval eq 'facing' ? 0 : 1 );
+                $self->_set_side_to_scan( $name, $newval );
             }
             when ('sided') {
                 $self->{$name} = $newval;
@@ -1025,7 +1064,12 @@ sub _flatbed_or_duplex_callback {
     my ($self) = @_;
     my $options = $self->get('available-scan-options');
     if ( defined $options ) {
-        if ( $options->flatbed_selected or $options->can_duplex ) {
+        if (
+            $options->flatbed_selected
+            or ( $options->can_duplex
+                and not $self->get('ignore-duplex-capabilities') )
+          )
+        {
             $self->{vboxx}->hide;
         }
         else {
@@ -1659,6 +1703,10 @@ sub set_paper {
         $options->by_name(SANE_NAME_SCAN_BR_Y)->{val}
     );
 
+    # forget the previous option info calls, as these are only interesting
+    # *whilst* setting a profile, and now we are starting from scratch
+    delete $self->{option_info};
+
     if ( not $paper_profile->num_backend_options ) {
         $self->hide_geometry($options);
         $self->{paper} = $paper;
@@ -2360,6 +2408,7 @@ sub add_current_scan_options {
 
 sub _set_option_profile {
     my ( $self, $profile, $next, $step ) = @_;
+    $self->set( 'cursor', 'wait' );
 
     if ( my $i = $next->($step) ) {
         my ( $name, $val ) = $profile->get_backend_option_by_index($i);
@@ -2452,6 +2501,7 @@ sub _set_option_profile {
             $self->get('current-scan-options'),
             $profile->{uuid}
         );
+        $self->set( 'cursor', 'default' );
     }
     return;
 }
