@@ -26,7 +26,7 @@ Readonly my $_1KB                         => 1024;
 Readonly my $ALL_PENDING_ZOMBIE_PROCESSES => -1;
 Readonly my $INFINITE_DOCUMENTS           => -1;
 
-our $VERSION = '2.6.2';
+our $VERSION = '2.6.6';
 
 my $EMPTY = q{};
 my $COMMA = q{,};
@@ -402,7 +402,7 @@ sub _scanadf {
                         and ( -s "out$id.pnm" ) >
                         50 )    ## no critic (ProhibitMagicNumbers)
                     {
-                        $size = Gscan2pdf::NetPBM::file_size_from_header(
+                        ($size) = Gscan2pdf::NetPBM::file_size_from_header(
                             "out$id.pnm");
                     }
                     else {
