@@ -16,7 +16,7 @@ BEGIN {
     use Exporter ();
     our ( $VERSION, @EXPORT_OK, %EXPORT_TAGS );
 
-    $VERSION = '2.6.2';
+    $VERSION = '2.6.6';
 
     use base qw(Exporter);
     %EXPORT_TAGS = ();              # eg: TAG => [ qw!name1 name2! ],
@@ -29,7 +29,7 @@ our @EXPORT_OK;
 
 my $COMMA = q{,};
 my $SPACE = q{ };
-my ($version);
+our ($UNPAPER_VERSION);
 
 sub new {
     my ( $class, $default ) = @_;
@@ -261,7 +261,7 @@ sub add_options {
     my $outpages = $self->add_widget( $vbox, $options, 'output-pages' );
     $combobl->signal_connect(
         changed => sub {
-            if ( $combobl->get_active == 0 ) {
+            if ( $self->get_option('layout') eq 'double' ) {
                 $outpages->set_range( 1, 2 );
             }
             else {
@@ -665,12 +665,12 @@ sub get_cmdline {
 }
 
 sub version {
-    if ( not defined $version ) {
-        $version =
+    if ( not defined $UNPAPER_VERSION ) {
+        $UNPAPER_VERSION =
           Gscan2pdf::Document::program_version( 'stdout', qr/([\d.]+)/xsm,
             [ 'unpaper', '--version' ] );
     }
-    return $version;
+    return $UNPAPER_VERSION;
 }
 
 1;
